<?php

declare(strict_types=1);

namespace hood\vk\api\methods;

use Exception;

use hood\accounts\vk;
use hood\vk\robots\robot;

/**
 * Сообщение
 *
 * @method public static function put(RobotAbstract $from, int $to, string $message, int $mode = 2) Отправить сообщение
 *
 * @see https://vk.com/dev/messages.send
 * @see https://vk.com/dev/messages.getById
 *
 * @package hood\vk\api
 * @author Arsen Mirzaev Tatyano-Muradovich <red@hood.su>
 *
 * @todo Доработать строгий режим отправки: проверку сообщения в беседе (не имеет ID сообщений)
 */
final class messages extends method
{
    /**
     * @param int $mode Режим отправки
     */
    protected int $mode = 1;

    /**
     * @param int|string|array $destination Получатель
     */
    protected int|string|array $destination;

    /**
     * @param string|null $text Текст
     */
    protected string|null $text = null;

    /**
     * @param array $attachments Вложения
     */
    protected array $attachments = [];

    /**
     * Создать сообщение
     *
     * @param robot $robot Робот
     */
    public function __construct(
        protected robot $robot
    ) {
    }

    /**
     * Добавить текст
     * 
     * @param string $text Текст
     */
    public function text(string $text): self
    {
        if (!isset($this->text)) {
            $this->text = $text;
        } else {
            $this->text .= $text;
        }

        return $this;
    }

    /**
     * Добавить Фото
     * 
     * @param $img Фото
     */
    public function image($img): self
    {
        //загрузить фото
        $id = $this->robot->photo()->get_photo($img);

        //добавить к вложениям
        $this->add($id);

        return $this;
    }

    /**
     * Добавить вложения
     * 
     * @param string|array $attachments Вложения
     */
    public function add(string|array $attachments): self
    {
        if (is_array($attachments)) {
            $this->attachments = array_merge($this->attachments, $attachments);
        } else {
            array_push($this->attachments, $attachments);
        }

        return $this;
    }

    /**
     * Отправить сообщение
     *
     * @param int|string|array $destination Получатель
     *
     * @see https://vk.com/dev/messages.send
     *
     * @return array Ответ сервера
     */
    public function send(int|string|array $destination): array
    {
        // Идентификатор сообщения
        $random_id = time();

        if ($this->mode = 1) {
            // Перемножение (по умолчанию)
            $random_id *= rand();
        }

        // Реиницилазиция
        $this->robot->api->reinit();

        // Цель отправки
        $this->robot->api->chooseDestination($this->destination);

        // Сообщение
        $this->robot->api['message'] = $this->message;

        // Идентификатор сообщения
        $this->robot->api['random_id'] = $random_id;

        // Фильтрация вложений
        $forward_messages = [];
        foreach ($this->attachments as &$attachment) {
            //var_dump($attachment);
            if (iconv_substr($attachment, 0,  7, "UTF-8") === 'message') {
                // Если среди вложений найдено сообщение для пересылки
                $forward_messages[] = $attachment;
                unset($attachment);
            }
        }

        if (!empty($forward_messages)) {
            // Если есть пересылаемые сообщения
            $this->robot->api['forward_messages'] = implode(',', $forward_messages);
        }

        //var_dump($attachments);
        if (!empty($this->attachments)) {
            // Если есть вложения
            $this->robot->api['attachment'] = implode(',', $this->attachments);
        }

        // Запрос
        $request = $this->robot->browser->request(method: 'POST', uri: 'messages.send', options: ['form_params' => $this->robot->api['settings']]);

        // Очистка
        //unset($settings);

        if ($this->mode >= 2) {
            // Если установлен режим 2 (усиленная проверка отправленного сообщения)

            if (!empty($request["response"])) {
                // Если пришел ID сообщения

                // Реиницилазиция
                $this->robot->api->reinit();

                // Запрашиваемые сообщения
                $this->robot->api['message_ids'] = $request["response"];

                // Запрос
                if ($this->robot->browser->post(uri: 'https://api.vk.com/method/messages.getById', options: $this->robot->api['settings'])['response']['count'] === 0) {
                    // Если сообщения не существует, то повторить отправку
                    $this->send($destination);
                }
            } else {
                // Что-то придумать :)
            }
        }

        return (array) $request;
    }

    public function __get($name)
    {
        return match ($name) {
            'text' => $this->text ?? throw new Exception('Текст не задан'),
            'attachments' => !empty($this->attachments) ? $this->attachments : throw new Exception('Вложения не заданы'),
            default => throw new Exception('Свойство не найдено: ' . $name)
        };
    }

    /**
     * Получить информацию о сообщении
     *
     * @return array Информация о сообщении
     */
    public function info(): array
    {
        //Инициализация
        //$settings = $this->init();
        $this->robot->api->init();

        // Цель отправки
        $this->robot->api->chooseDestination($this->destination);

        // Сообщение
        $this->robot->api['message'] = $this->message;

        // Режим отправки
        $settings['mode'] = $this->mode; //!!!!!!!!!!!!!!!!!!!!!!

        // Фильтрация вложений
        $forward_messages = [];
        foreach ($this->attachments as &$attachment) {
            if (iconv_substr(str: $attachment, offset: 0, length: 7, charset: "UTF-8") === 'message') {
                // Если среди вложений найдено сообщение для пересылки

                $forward_messages[] = $attachment;
                unset($attachment);
            }
        }

        if (!empty($forward_messages)) {
            // Если есть пересылаемые сообщения

            $this->robot->api['forward_messages'] = implode(',', $forward_messages);
        }

        if (!empty($attachments)) {
            // Если есть вложения

            $this->robot->api['attachment'] = implode(',', $this->attachments);
        }

        return $settings;
    }
}
