<?php

declare(strict_types=1);

use PHPUnit\Framework\TestCase;

use hood\vk\core;
use hood\vk\robots\robot;
use hood\vk\tests\settings;

use hood\accounts\vk as account;

/**
 * @testdox Сообщения
 */
final class messagesTest extends TestCase
{
    use settings;

    /**
     * @var core $core Ядро фреймворка
     */
    private static core $core;

    /**
     * @var account $account Аккаунт
     */
    protected static account $account;

    /**
     * @var robot $robot Робот
     */
    private static robot $robot;

    /**
     * @testdox Инициализация аккаунта
     * @beforeClass
     */
    public function testAccountInit(): void
    {
        // Проверка входных данных
        if (empty(self::$project_id)) {
            self::$markTestSkipped('Не найден идентификатор проекта');
        }

        // Инициализация аккаунта
        self::$account = new account(empty(self::$id) ? rand(0, 10) : self::$id, __DIR__ . '../../../accounts');

        // Запись режима SSL-протокола
        self::$account->ssl = self::$ssl ?? true;

        if (empty(self::$key)) {
            // Если не указан ключ

            // Проверка входных данных
            if (empty(self::$login)) {
                self::$markTestSkipped('Не найден входной аккаунта');
            } else if (empty(self::$password)) {
                self::$markTestSkipped('Не найден пароль аккаунта');
            }

            // Деаутентификация (на всякий случай)
            self::$account->deauth();

            // Аутентификация и генерация ключа
            self::$key = self::$account->auth(self::$login, self::$password)->key(self::$project_id);
        }

        $this->assertNotNull(self::$key, 'Ошибка при инициализации ключа аккаунта');
    }

    /**
     * @testdox Деинициализация аккаунта
     * @afterClass
     */
    public static function testAccountDeinit(): void
    {
        // Инициализация аккаунта
        self::$account = new account(empty(self::$id) ? rand(0, 10) : self::$id, __DIR__ . '../../../accounts');

        // Деаутентификация
        self::$account->deauth();
    }

    /**
     * @testdox Инициализация робота
     * @before
     */
    public function testRobotGroupInit(): void
    {
        // Инициализация ядра
        self::$core = core::init();

        // Сохранение количества роботов
        $count = self::$core->robots;

        // Инициализация робота
        self::$robot = self::$core->group(empty(self::$group_id) ? rand(0, 10) : self::$group_id);

        // Проверка
        $this->assertEquals(self::$core->robots, $count + 1, 'Ошибка при инициализации робота');
    }

    /**
     * @testdox Деинициализация робота
     * @after
     */
    public function testRobotGroupDeinit(): void
    {
        // Очистка реестра
        self::$core->delete();

        // Проверка
        $this->assertEmpty(self::$core->get(self::$robot->id), 'Ошибка при деинициализации робота');
    }

    /**
     * @testdox Чтение текста
     */
    public function testReadText(): void
    {
        $this->expectExceptionMessage('Текст не задан');

        self::$robot->message()->text;
    }

    /**
     * @testdox Чтение вложений
     */
    public function testReadAttachments(): void
    {
        $this->expectExceptionMessage('Вложения не заданы');

        self::$robot->message()->attachments;
    }


    /**
     * @testdox Добавление текста
     */
    public function testWriteText() //: hood\vk\api\methods\messages
    {
        //self::$robot->key(self::$group_key);
        return self::$robot->message()->text('text');
    }

    /**
     * @testdox Повторное добавление текста
     * 
     * @depends testWriteText
     */
    public function testWriteTextWhenHeIsAlreadyWrited($messages)
    {
        $messages->text('text');

        $this->assertSame('text' . 'text', $messages->text);

        return $messages;
    }

    /**
     * @testdox Добавление массива вложений
     * 
     * @depends testWriteTextWhenHeIsAlreadyWrited
     */
    public function testAddAttAttachments($messages)
    {
        return $messages->add(['text']);
    }

    /**
     * @testdox Повторное добавление текста вложения
     * 
     * @depends testAddAttAttachments
     */
    public function testAddAttachmentsWhenTheyIsAlreadyWrited($messages)
    {
        //временный код
        $this->assertSame(['text', 'text'], $messages->add('text')->attachments);

        return $messages;
    }

    /**
     * @testdox Отправка сообщения
     * 
     * @depends testAddAttachmentsWhenTheyIsAlreadyWrited
     */
    //public function testSend($messages)
    //{
        //Проверка выбрса исключения
        //$this->expectExceptionMessage('Ключ не инициализирован');
        
        //Отправка по идентификатору
        //$messages->send(self::$peer_id);

        //Отправка по идентификаторам
        //$messages->send(self::$peer_ids);

        //Отправка по домену
        //$messages->send(self::$domain);

    //}
}
